PAYROLL_USECOLS: list = [
    "Company Short Name",
    "Pay Statement Type",
    "Employee Id",
    "Name",
    "Base Hourly Rate",
    "E Regular Hours",
    "E Overtime Hours",
    "E Vacation Hours",
    "E Charge Tips Amount",
    "E Opp Pay Week 1 Amount",
    "E Opp Pay Week 2 Amount",
    "D EE Uniform Amount",
    "Default Job",
    "Default Work Location"
]

TIMESHEET_USECOLS: list = [
    "name",
    "minorJob",
    "minutesWorked",
    "startDate",
    "endDate",
    "regularMinutes",
    "overtimeMinutes"
]

TIPSHEET_USECOLS: list = ["firstName", "lastName", "chargeTips", "cashTips"]



import numpy as np
import pandas as pd

from .primitives import PAYROLL_USECOLS, TIMESHEET_USECOLS, TIPSHEET_USECOLS
from .process import DocumentProcessor
from .utils import handle_data_file, save_df_as, convert_minutes_to_hours, create_name_column


def timesheet(df):
    doc = DocumentProcessor(
        actions={i: "sum" for i in ["regularMinutes", "overtimeMinutes", "minutesWorked"]},
        groups=["minorJob", "name"],
        pattern=".artifacts/files/EmployeeTimesheet_*",
        query="CostAnalysisDataset_GenerateCostAnalysisReport_LabourCosts",
        usecols=TIMESHEET_USECOLS
    )
    times = convert_minutes_to_hours(pd.DataFrame(doc.clean()))
    for i, j, k in zip(times.name, times.minorJob, range(times.shape[0])):
        df.loc[(df.Name.str.lower() == i) & (df["Default Job"] == j), "E Regular Hours"] = times.regularHours[k]
        df.loc[(df.Name.str.lower() == i) & (df["Default Job"] == j), "E Overtime Hours"] = times.overtimeHours[k]
    return df


def tipsheet(df):
    doc = DocumentProcessor(
        pattern=".artifacts/files/ServerTipDeclarations_*",
        query="ServerTipDeclarationDataSet_GenerateTipDeclaration",
        usecols=TIPSHEET_USECOLS
    )
    tips = create_name_column(pd.DataFrame(doc.clean()))
    print(tips.columns)
    for i, j in enumerate(tips["name"]):
        try:
            df.loc[df.Name.str.lower() == j, "E Charge Tips Amount"] = tips["chargeTips"][i]
        except KeyError:
            print("Key Error: Skipping row {} of {}".format(i, tips.shape[0]))
    return df


class Payroll(object):
    data: dict
    usecols: list = PAYROLL_USECOLS

    def __init__(self, filepath):
        self.path: str = filepath
        self.run()

    def __call__(self, *args, **kwargs):
        df = pd.DataFrame(self.data)
        if kwargs["save_as"]:
            save_df_as(df, kwargs["save_as"])
        return df

    def run(self) -> pd.DataFrame:
        df = handle_data_file(self.path, usecols=self.usecols)
        df = timesheet(df)
        df = tipsheet(df)
        df.replace(np.nan, 0.0, inplace=True)
        self.data = df.to_dict()
        return df


if __name__ == "__main__":
    tp = Payroll(".artifacts/files/Payroll.xls")



from typing import List

import numpy as np
import pandas as pd

from tads.utils import collect_dfs, save_df_as


class DocumentProcessor(object):
    data: dict

    def __init__(self, pattern, query, usecols, actions=None, groups=None):
        self.actions: dict = actions
        self.groups: List[str] = groups
        self.pattern: str = pattern
        self.query: str = query
        self.usecols: List[str] = usecols

    def __call__(self, *args, **kwargs):
        self.data = self.clean()

        return self.data[:].to_dict()

    def aggregate(self, **kwargs) -> list:
        return collect_dfs(self.pattern, self.query, **kwargs)

    def combine(self, ignore_index: bool = True, **kwargs) -> dict:
        return pd.concat(self.aggregate(), ignore_index=ignore_index, **kwargs).to_dict()

    def clean(self) -> dict:
        df = pd.DataFrame(self.combine())
        df = df.drop([i for i in df.columns if i not in self.usecols], axis=1)
        df.replace('', np.nan, inplace=True)
        if self.groups:
            df = df.groupby(self.groups, as_index=False)
        if self.actions:
            df = df.agg(self.actions)
        return df.to_dict()

    def export(self, save_as: str):
        save_df_as(pd.DataFrame(self.data), save_as)

    def __repr__(self):
        return f""""{self.clean()}"""



from glob import glob

import pandas as pd


def collect_dfs(pattern, query, **kwargs) -> list:
    return [handle_data_file(f, skiprows=find_row(f, query), **kwargs) for f in find_similar(pattern)]


def convert_minutes_to_hours(data: pd.DataFrame) -> pd.DataFrame:
    for c in data.columns:
        if "minutes" in c.lower():
            data[f"{c.lower().split('minutes')[0]}Hours"] = data[c] / 60.0
    return data


def create_name_column(data: pd.DataFrame) -> pd.DataFrame:
    data["name"] = data["firstName"] + " " + data["lastName"]
    return data


def create_path(base, *args) -> str: return "{}/{}".format(base, "/".join(args))


def find_row(path: str, query: str) -> int:
    try:
        print("Searching lines...")
        return search_file(path, query)[-1] + 1
    except (pd.errors.ParserError, UnicodeDecodeError):
        print("Searching dataframe...")
        return search_excel(path, query)[0] + 2


def find_similar(pattern: str, **kwargs) -> list: return glob(pattern, **kwargs)


def handle_data_file(path, **kwargs) -> pd.DataFrame:
    if "csv" in path:
        return pd.read_csv(path, **kwargs)
    elif "json" in path:
        return pd.read_json(path, **kwargs)
    elif "xls" or "xlsx" in path:
        return pd.read_excel(path, **kwargs)
    else:
        print("Failed to find an operational match for the provided input...")
        raise IOError


def index_of(query: str, *args): return [i for i, j in enumerate(args) if query in j]


def identify_table_matches(path: str, query: str) -> list:
    df = handle_data_file(path)
    return df.index[df[df.columns.to_list()[0]] == query].to_list()


def save_df_as(df, save_as: str):
    if "csv" in save_as:
        df.to_csv(save_as, index=False)
    elif "xls" or "xlsx" in save_as:
        df.to_excel(save_as, index=False)
    else:
        print("IOError: Failed to match save path to a defined operation...")
        raise IOError


def search_file(path: iter, query: str) -> list:
    return [i for i, j in enumerate(open(path, "r")) if query in j or query == j]


def search_excel(path: str, query: str) -> list:
    df = handle_data_file(path)
    return df.index[df[df.columns.to_list()[0]] == query].to_list()



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['tads']

package_data = \
{'': ['*']}

install_requires = \
['numpy', 'openpyxl', 'pandas', 'python-multipart', 'xlrd']

setup_kwargs = {
    'name': 'tads',
    'version': '0.1.2',
    'description': 'Tads is reserved for ...',
    'long_description': None,
    'author': 'FL03',
    'author_email': 'jo3mccain@icloud.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<3.11',
}


setup(**setup_kwargs)
